import time
import numpy as np
from srsgui import Task
from srsgui.task.inputs import InstrumentInput, IntegerInput

from srsinst.rga.plots.timeplot import TimePlot
from srsinst.rga.plots.analogscanplot import AnalogScanPlot
from srsinst.rga.plots.histogramscanplot import HistogramScanPlot

from srsinst.uga import Keys, get_uga, get_rga


class UGAMultiplotTask(Task):
    """
    Run multiple plots for UGA
    """
    InstrumentName = 'uga to monitor'

    input_parameters = {
        InstrumentName: InstrumentInput(),
    }

    additional_figure_names = ['analog_scan', 'histogram_scan', 'rga_analog_scan']

    def setup(self):
        self.logger = self.get_logger(__name__)

        self.instrument_name_value = self.get_input_parameter(self.InstrumentName)
        self.uga = get_uga(self, self.instrument_name_value)
        if self.uga.rga.state != Keys.On:
            raise ValueError('UGA RGA is off')
        self.uga.rga.scan.set_parameters(1, 50, 5, 10)

        self.rga = get_rga(self, 'rga')

        self.ax = self.get_figure().subplots(nrows=1, ncols=2, sharex=True)
        self.ax_analog = self.get_figure('analog_scan').add_subplot(111)
        self.ax_histogram = self.get_figure('histogram_scan').add_subplot(111)
        self.ax_rga_analog = self.get_figure('rga_analog_scan').add_subplot(111)

        self.pressure_plot = TimePlot(self, self.ax[0], 'Pressure',
            ['IG pressure', 'PG pressure', 'CM pressure'])
        self.pressure_plot.ax.set_yscale('log')

        self.temperature_plot = TimePlot(self, self.ax[1], 'Temperature',
            ['Chamber Temperature', 'Elbow Temperature', 'Sample Inlet Temperature',
             'Capillary Temperature', 'Turbo Pump Temperature'])

        self.uga.rga.scan.set_parameters(1, 50, 5, 10)
        self.analog_scan_plot = AnalogScanPlot(self, self.ax_analog, self.uga.rga.scan, 'Analog Scan')
        self.analog_scan_plot.set_conversion_factor(
            self.uga.rga.pressure.get_partial_pressure_sensitivity_in_torr(), 'Torr')

        self.uga.rga.scan.set_parameters(10, 45, 3, 10)
        self.histogram_scan_plot = HistogramScanPlot(self, self.ax_histogram, self.uga.rga.scan, 'Histogram Scan')

        self.rga.scan.set_parameters(1, 50, 3, 10)
        self.rga_analog_scan_plot = AnalogScanPlot(self, self.ax_rga_analog,
                                                   self.rga.scan, 'RGA analog')

    def test(self):
        while True:
            if not self.is_running():
                break

            self.display_device_info(device_name=self.instrument_name_value, update=True)

            self.pressure_plot.add_data([self.uga.ig.get_pressure(), self.uga.rp.get_pressure(),
                         self.uga.bp.get_pressure()])

            self.temperature_plot.add_data([self.uga.temperature.chamber, self.uga.temperature.elbow,
                 self.uga.temperature.sample_inlet, self.uga.temperature.capillary,
                 self.uga.temperature.turbo_pump], True)

            self.rga.scan.set_parameters(1, 50, 3, 10)
            self.rga_analog_scan_plot.reset()
            self.rga.scan.get_analog_scan()

            self.uga.rga.scan.set_parameters(10, 45, 3, 10)
            self.histogram_scan_plot.reset()
            self.uga.rga.scan.get_histogram_scan()

            self.uga.rga.scan.set_parameters(1, 50, 5, 10)
            self.analog_scan_plot.reset()
            self.uga.rga.scan.get_analog_scan()

    def cleanup(self):
        self.analog_scan_plot.cleanup()
        self.histogram_scan_plot.cleanup()
        self.rga_analog_scan_plot.cleanup()
        self.temperature_plot.cleanup()
        self.pressure_plot.cleanup()



# Configuration file for the Sphinx documentation builder.
#
# For the full list of built-in configuration values, see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html


# -- Path setup --------------------------------------------------------------
# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
sys.path.insert(0, os.path.abspath('../../uga100'))
print(os.path.abspath('../../uga100'))

from rga import __version__

# -- Project information -----------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information

project = 'uga'
copyright = '2022, Stanford Research Systems'
author = 'Chulhoon Kim'
version = __version__

# -- General configuration ---------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#general-configuration



extensions = ['sphinx.ext.autodoc',
              'sphinx.ext.napoleon',
             ]

autodoc_member_order ="bysource"

autodoc_default_options = {
    'members': True,
    # The ones below should be optional but work nicely together with
    # example_package/autodoctest/doc/source/_templates/autosummary/class.rst
    # and other defaults in sphinx-autodoc.
    'show-inheritance': True,
    'inherited-members': True,
    'no-special-members': True,
}


templates_path = ['_templates']
exclude_patterns = []



# -- Options for HTML output -------------------------------------------------
# https://www.sphinx-doc.org/en/master/usage/configuration.html#options-for-html-output


html_theme = 'sphinx_rtd_theme'
#html_theme = 'alabaster'

html_static_path = ['_static']




from srsgui import Component, Instrument

from srsgui import Command, GetCommand,\
                   BoolCommand, BoolGetCommand,\
                   IntCommand, IntGetCommand, IntSetCommand,\
                   FloatCommand, FloatSetCommand, FloatGetCommand, \
                   DictCommand, DictGetCommand

from srsgui import IndexCommand, IndexGetCommand, \
                   IntIndexCommand, IntIndexGetCommand, \
                   BoolIndexCommand, BoolIndexGetCommand,\
                   FloatIndexCommand, FloatIndexGetCommand
from srsinst.rga import RGA100
from .keys import Keys


class Mode(Component):
    ModeDict = {
        Keys.Off:         1,
        Keys.Start:       3,
        Keys.Stop:        4,
        Keys.Ready:       6,
        Keys.Sleep:       7,
        Keys.Idle:        8,
        Keys.LeakTest:   10,
        Keys.SystemBake: 12,
        Keys.Manual:     13
    }

    StateDict = {
        Keys.Off: 0,
        Keys.On: 1,
        Keys.Idle: 2,
        Keys.TurningOn3: 3,
        Keys.TurningOn4: 4,
        Keys.TurningOn5: 5,
        Keys.TurningOff6: 6,
        Keys.TurningOff7: 7,
        Keys.TurningIdle8: 8,
        Keys.TurningIdle9: 9,
        Keys.TurningIdle10: 10,
        Keys.Error: 12,
    }

    OffOnDict = {
        Keys.Off: 0,
        Keys.On:  1,
    }

    OffOnIdleDict = {
        Keys.Off:  0,
        Keys.On:   1,
        Keys.Idle: 2,
    }

    state = DictGetCommand('ZMOD', ModeDict)
    bake = BoolCommand('ZMBK')

    leak_test = BoolCommand('ZMLT')
    leak_test_mass = IntCommand('ZPLM', 'AMU')

    def start(self):
        self.comm.send('ZMST')

    def stop(self):
        self.comm.send('ZMSP')
        
    def sleep(self):
        self.comm.send('ZMSL')

    allow_run_button = [start, stop, sleep]


class BypassPump(Component):
    state = DictCommand('ZCBP', Mode.OffOnDict, Mode.StateDict)
    on_power = IntCommand('ZPBO', '%')

    def get_pressure(self):
        return self._parent.pressure.values[Keys.Bypass] * 1e-6

    
class RoughingPump(Component):
    state = DictCommand('ZCRP', Mode.OffOnIdleDict, Mode.StateDict)
    on_power = IntCommand('ZPRO', '%')
    idle_power = IntCommand('ZPRI', '%')

    def get_pressure(self):
        return self._parent.pressure.values[Keys.Roughting] * 1e-6

    
class TurboPump(Component):
    state = DictCommand('ZCTP', Mode.OffOnIdleDict, Mode.StateDict)
    speed = IntGetCommand('ZQHZ', 'Hz')
    current = FloatGetCommand('ZQCU', 'mA')
    temperature = IntGetCommand('ZQTT', '°C')

    def get_pressure(self):
        if self._parent.ig.state == Keys.On:
            return self._parent.pressure.values[Keys.Chamber] * 1e-12
        else:
            return 0.0


class BypassValve(Component):
    state = DictCommand('ZCBV', Mode.OffOnDict, Mode.StateDict)


class SampleValve(Component):
    state = DictCommand('ZCSV', Mode.OffOnDict, Mode.StateDict)
    auto = BoolCommand('ZPAS')


class VentValve(Component):
    state = DictCommand('ZCVV', Mode.OffOnDict, Mode.StateDict)
    auto = BoolCommand('ZPAV')


class RGA(RGA100):
    state = DictCommand('ZCRG', Mode.OffOnDict, Mode.StateDict)

    def __init__(self, parent):
        super().__init__()

        self._parent = parent
        self._parent._children.append(self)
        self._add_parent_to_index_commands()
        self.comm = parent.comm
        self.update_components()

        if self.comm.is_connected():
            if self.state == Keys.On:
                self.check_id()
   

class IonGauge(Component):
    """
    IonGauge component has two commands and one method:

    The state command turn on and off the ion gauge.
    The ion gauge can be turned on, only when the turbo pump is on or idle.

    The ion gauge has two filaments and if one is broken, you can select the other one.

    The method get_pressure() return the ion gauge pressure reading in Torr.
    """
    StateDict = {
        Keys.Off: 0,
        Keys.On: 1,
        Keys.Degas: 2
    }
    FilamentDict = {
        Keys.Filament1: 0,
        Keys.Filament2: 1
    }

    state = DictCommand('ZCIG', StateDict, Mode.StateDict)
    filament = DictCommand('ZPFL', FilamentDict)
    
    def get_pressure(self):
        """ Get the IG pressure in Torr"""
        return self._parent.pressure.values[Keys.IG] * 1e-12

    
class Heaters(Component):
    ModeDict = {
        Keys.Off: 0,
        Keys.BakeOn: 1,
        Keys.SampleOn: 2
    }
    HeaterDict = {
        Keys.Elbow: 0,
        Keys.Chamber: 1,
        Keys.SampleLine: 2,
        Keys.Capillary: 3
    }
    BakeHeaterDict = {
        Keys.Elbow: 0,
        Keys.Chamber: 1,
    }

    state = DictCommand('ZCHT', ModeDict, Mode.StateDict)
    bake_time = IntCommand('ZPBT', 'hr.')
    bake_time_remained = IntGetCommand('ZQBR', 'min.')
    bake_temperature = IntIndexCommand('ZPTB', 1, 0, BakeHeaterDict, '°C')
    sample_temperature = IntIndexCommand('ZPTH', 3, 0, HeaterDict, '°C')


class Temperature(Component):
    """
    Temperature component contains temperature measurements from 5 sensors.
    If the temperature calue us 255, the sensor is not working properly.
    """
    turbo_pump = IntGetCommand('ZQTT', '°C')
    elbow = IntGetCommand('ZQTA', '°C')
    chamber = IntGetCommand('ZQTB', '°C')
    sample_inlet = IntGetCommand('ZQTC', '°C')
    capillary = IntGetCommand('ZQTD', '°C')


class MultipleInlet(Component):
    channel = IntCommand('ZCMI')


class Pressure(Component):
    """
    Pressure component has two commands:

         Values are pressure measured with gauges:

            Pressure from Pirani and CM gauges are in the unit of uTorr (1e-6 Torr).
            pressure from IG is in the unit of pTorr (1e-12 Torr) regardless of the display unit

         Display_unit is to select the pressure unit for the front panel display.

            Reply from values command always in the unit of Torr.
    """
    GaugeDict = {
        Keys.Pirani: 0,
        Keys.Roughting: 0,
        Keys.CM: 2,
        Keys.Bypass: 2,
        Keys.IG: 3,
        Keys.Chamber: 3
    }

    UnitDict = {
        Keys.Torr: 0,
        Keys.Pascal: 1,
        Keys.MilliBar: 2,
        Keys.Bar: 3
    }
    values = IntIndexGetCommand('ZQAD', 3, 0, GaugeDict)
    display_unit = DictCommand('ZPPU', UnitDict)



class Ethernet(Component):
    mac_address = GetCommand('ZQMC')
    ip_address = Command('ZPIP')
    gateway = Command('ZPGW')
    subnet_mask = Command('ZPSM')
    login = Command('ZPNM')
    password = Command('ZPPW')
    full_duplex = BoolCommand('ZPDU')
    speed = IntCommand('ZPSP')
    timeout = IntCommand('ZPTO', 's')
    """A new timeout requires a power cycle to be effective."""
    exclude_capture = [login, password]


class Status(Component):
    StateBitDict = {
        15: ('Error', 'Occured', 'None'),
        14: (None, 'Power', 'On', 'Off'),  # always on when communicating
        13: ('Auto Mode', 'On', 'Off'),
        12: (None, None, None),
        11: ('Bake Mode', 'On', 'Off'),
        10: ('Heaters', 'On', 'Off'),
        9:  ('Vent Valve', 'Open', 'Closed'),
        8:  ('Ion Gauge', 'On', 'Off'),
        7:  ('RGA', 'On', 'Off'),
        6:  ('Sample Valve', 'Open', 'Closed'),
        5:  ('Bypass Valve', 'Open', 'Closed'),
        4:  ('TP Idle', 'On', 'Off'),
        3:  ('Turbo Pump', 'On', 'Off'),
        2:  ('RP Idle', 'On', 'Off'),
        1:  ('Roughing Pump', 'On', 'Off'),
        0:  ('Bypass Pump', 'On', 'Off')
    }

    pressure_interlock = BoolCommand('ZCPC')
    speaker_volume = IntCommand('ZCVL')
    
    id_string = GetCommand('ZQID')
    firmware_version = GetCommand('ZQFV')
    serial_number = GetCommand('ZQSN')
    baud_rate = IntCommand('ZPBA')

    states = IntGetCommand('ZBST')
    changed = IntGetCommand('ZBCT')
    changing = IntGetCommand('ZBTT')
    error = DictGetCommand('ZERR', Keys.ErrorMessageDict)
    error_message = IndexGetCommand('ZEDS', index_max=126)

    exclude_capture = [error_message]

    def get_status_text(self):
        out_buffer = ''
        error_buffer = 'Errors: '
        item_line_format = '{}: {} \n'
        
        states = self.states
        changed = self.changed
        changing = self.changing        

        out_buffer += item_line_format.format('Mode', self._parent.mode.state)
        
        for ind in Status.StateBitDict:
            item_name = Status.StateBitDict[ind][0]
            on_tag = Status.StateBitDict[ind][1]
            off_tag = Status.StateBitDict[ind][2]
            if item_name in (None, 'TP Idle', 'RP Idle'):
                continue

            mask = 1 << ind                
            if item_name == 'Error':
                if not states & mask:
                    error_buffer += off_tag
                    continue

                while True:
                    error = self.error
                    if error == Keys.OK:
                        break
                    error_buffer += '{}\n'.format(error)
                continue

            if item_name in ('Turbo Pump', 'Roughing Pump'):
                idle_mask = mask << 1
                if changing & idle_mask:                
                    text = item_line_format.format(item_name, 'Changing')
                elif states & idle_mask:
                    text = item_line_format.format(item_name, 'Idle')
                elif changing & mask:
                    text = item_line_format.format(item_name, 'Changing')
                elif states & mask:
                    text = item_line_format.format(item_name, on_tag)
                else:
                    text = item_line_format.format(item_name, off_tag)
                out_buffer += text
                continue
                
            if changing & mask:
                text = item_line_format.format(item_name, 'Changing')
            elif states & mask:
                text = item_line_format.format(item_name, on_tag)
            else:
                text = item_line_format.format(item_name, off_tag)
            out_buffer += text
            
        out_buffer += error_buffer   
        return out_buffer



import time
import numpy as np
from srsgui import Task
from srsgui import InstrumentInput, IntegerInput
from srsinst.rga.plots.timeplot import TimePlot

from srsinst.uga import get_uga


class UGAStateMonitorTask(Task):
    """Monitor UGA pressure and temperature 
    """
    InstrumentName = 'uga to monitor'
    UpdatePeriod = 'update period'
    input_parameters = {
        InstrumentName: InstrumentInput(),
        UpdatePeriod: IntegerInput(2, ' s', 1, 60, 1)
    }

    def setup(self):
        self.logger = self.get_logger(__name__)
        self.params = self.get_all_input_parameters()
        self.uga = get_uga(self, self.params[self.InstrumentName])

        self.ax = self.figure.subplots(nrows=1, ncols=2, sharex=True)

        self.pressure_plot = TimePlot(self, self.ax[0], 'Pressure',
            ['IG pressure', 'PG pressure', 'CM pressure'])
        self.pressure_plot.ax.set_yscale('log')

        self.temperature_plot = TimePlot(self, self.ax[1], 'Temperature',
            ['Chamber Temperature', 'Elbow Temperature', 'Sample Inlet Temperature',
             'Capillary Temperature', 'Turbo Pump Temperature'])

    def test(self):
        while True:
            if not self.is_running():
                break

            self.display_device_info(device_name=self.params[self.InstrumentName], update=True)

            self.pressure_plot.add_data([self.uga.ig.get_pressure(), self.uga.rp.get_pressure(),
                         self.uga.bp.get_pressure()])

            self.temperature_plot.add_data([self.uga.temperature.chamber, self.uga.temperature.elbow,
                         self.uga.temperature.sample_inlet, self.uga.temperature.capillary,
                         self.uga.temperature.turbo_pump], True)

            time.sleep(self.params[self.UpdatePeriod])

    def cleanup(self):
        pass




from srsinst.uga.instruments.uga100.uga import UGA100
from srsinst.uga.instruments.get_instruments import get_rga, get_uga
from srsinst.uga.instruments.uga100.keys import Keys

__version__ = "0.0.1"  # Global version number


import logging
from srsgui import Task
from srsinst.rga import RGA100

from srsinst.uga.instruments.uga100.uga import UGA100

"""
This is a name used in .taskConfig file for an instrument definition
"""

logger = logging.getLogger(__name__)


def get_rga(task: Task, name=None) -> RGA100:
    """
    Instead of using task.get_instrument() directly in a Task subclass,
    Defining a wrapper function that returns a specific instrument
    return type will help a context-sensitive editors to display
    attributes available for the instrument class.
    If name is None, the first instrument in the .taskconfig file is assumed.
    """
    if name is None:
        inst = list(task.inst_dict.values())[0]
    else:
        inst = task.get_instrument(name)
    if issubclass(type(inst), RGA100):
        return inst
    elif issubclass(type(inst), UGA100):
        return inst.rga
    else:
        raise TypeError('{} is not a RGA100 instance'.format(name))


def get_uga(task: Task, name=None) -> UGA100:
    inst = task.get_instrument(name)

    if not issubclass(type(inst), UGA100):
        raise TypeError('{} is not a UGa100 instance'.format(name))
    return inst
